import { expect, test } from 'vitest'
import { extractAllowRules } from "../src/background/utils/utils.js";

test("extract allow - empty file", () => {
  const rules = new Set();
  const input = [];
  const expected = new Set();
  extractAllowRules(rules, input);
  expect(rules).toEqual(expected);
});

test("extract allow - not valid", () => {
  const rules = new Set();
  const input = ["hello"];
  const expected = new Set();
  extractAllowRules(rules, input);
  expect(rules).toEqual(expected);
});

test("extract allow - not valid with rule", () => {
  const rules = new Set();
  const input = ["hello", "@@||nytimes.com/*^$document"];
  const expected = new Set();
  extractAllowRules(rules, input);
  expect(rules).toEqual(expected);
});

test("extract allow - empty preferences", () => {
  const rules = new Set();
  const input = ["# Adblock Plus preferences"];
  const expected = new Set();
  extractAllowRules(rules, input);
  expect(rules).toEqual(expected);
});
test("extract allow - minimum", () => {
  const rules = new Set();
  const input = ["# Adblock Plus preferences", "@@||nytimes.com/*^$document"];
  const expected = new Set().add("@@||nytimes.com/*^$document");
  extractAllowRules(rules, input);
  expect(rules).toEqual(expected);
});

test("extract allow - complete", () => {
  const rules = new Set();
  const input = [
    "# Adblock Plus preferences",
    "version=5",
    "[Subscription]",
    "url=~user~818073",
    "defaults=whitelist",
    "[Subscription filters]",
    "@@||nytimes.com/*^$document",
  ];
  const expected = new Set().add("@@||nytimes.com/*^$document");
  extractAllowRules(rules, input);
  expect(rules).toEqual(expected);
});

test("extract allow - complete duplicates", () => {
  const rules = new Set();
  const input = [
    "# Adblock Plus preferences",
    "version=5",
    "[Subscription]",
    "url=~user~818073",
    "defaults=whitelist",
    "[Subscription filters]",
    "@@||nytimes.com/*^$document",
    "@@||nytimes.com/*^$document",
  ];
  const expected = new Set().add("@@||nytimes.com/*^$document");
  extractAllowRules(rules, input);
  expect(rules).toEqual(expected);
});

test("extract allow - complete section duplicates", () => {
  const rules = new Set();
  const input = [
    "# Adblock Plus preferences",
    "version=5",
    "[Subscription]",
    "url=~user~818073",
    "defaults=whitelist",
    "[Subscription filters]",
    "@@||nytimes.com/*^$document",
    "[Subscription]",
    "url=~user~818073",
    "defaults=whitelist",
    "[Subscription filters]",
    "@@||nytimes.com/*^$document",
    "@@||nytimes.com/*^$document",
  ];
  const expected = new Set().add("@@||nytimes.com/*^$document");
  extractAllowRules(rules, input);
  expect(rules).toEqual(expected);
});
